"""
x402 Extensions

This package contains extension helpers for the x402 protocol.

Extensions are optional metadata that can be attached to payment-protected
resources to enable additional functionality beyond basic payment requirements.

Currently available extensions:
- nevermined: Payment management using Nevermined's credit system
"""

__all__ = []
